"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCardanoCli = void 0;
const child_process_1 = require("child_process");
const util_1 = require("@cardano-graphql/util");
const isEraMismatch = (errorMessage, era) => {
    return errorMessage.includes('EraMismatch') ||
        errorMessage.includes(`The attempted local state query does not support the ${util_1.capitalizeFirstChar(era)} protocol`);
};
function createCardanoCli(cardanoCliPath, genesis, jqPath) {
    const networkArg = genesis.networkId === '1'
        ? '--mainnet'
        : `--testnet-magic ${genesis.networkMagic.toString()}`;
    const query = (queryName, options) => {
        return new Promise((resolve, reject) => {
            for (const era of util_1.knownEras) {
                const command = [cardanoCliPath, 'query', queryName, networkArg];
                if (options === null || options === void 0 ? void 0 : options.withEraFlag) {
                    command.push(`--${era}-era`);
                }
                if (options === null || options === void 0 ? void 0 : options.jqOperation) {
                    command.push('|', jqPath, options === null || options === void 0 ? void 0 : options.jqOperation);
                }
                child_process_1.exec(command.join(' '), (error, stdout, stderr) => {
                    if (error !== null || stderr.toString() !== '') {
                        if (!isEraMismatch(stderr.toString(), era)) {
                            return reject(new Error(stderr.toString()));
                        }
                    }
                    else {
                        return resolve(JSON.parse(stdout));
                    }
                });
            }
        });
    };
    return {
        getProtocolParams: () => query('protocol-parameters', {
            withEraFlag: true
        }),
        getTip: () => query('tip'),
        submitTransaction: (filePath) => {
            return new Promise((resolve, reject) => {
                child_process_1.exec(`${cardanoCliPath} transaction submit --tx-file ${filePath} ${networkArg}`, (error, _stdout, stderr) => {
                    if (error !== null || stderr.toString() !== '') {
                        return reject(new Error(stderr.toString()));
                    }
                    return resolve();
                });
            });
        }
    };
}
exports.createCardanoCli = createCardanoCli;
//# sourceMappingURL=CardanoCli.js.map