"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CardanoNodeClient = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const p_retry_1 = __importDefault(require("p-retry"));
const util_1 = __importStar(require("@cardano-graphql/util"));
const temp_write_1 = __importDefault(require("temp-write"));
const ts_log_1 = require("ts-log");
const util_2 = require("./util");
const fileTypeFromEra = (era) => {
    switch (era) {
        case 'mary':
            return 'Tx MaryEra';
        case 'allegra':
            return 'Tx AllegraEra';
        case 'shelley':
            return 'TxSignedShelley';
        default:
            throw new Error(`Transaction not submitted. ${era} era not supported.`);
    }
};
const isEraMismatch = (errorMessage) => errorMessage.includes('DecoderErrorDeserialiseFailure') ||
    errorMessage.includes('The era of the node and the tx do not match');
class CardanoNodeClient {
    constructor(cardanoCli, lastConfiguredMajorVersion, logger = ts_log_1.dummyLogger) {
        this.cardanoCli = cardanoCli;
        this.lastConfiguredMajorVersion = lastConfiguredMajorVersion;
        this.logger = logger;
    }
    getTip() {
        return __awaiter(this, void 0, void 0, function* () {
            const tip = yield this.cardanoCli.getTip();
            this.logger.debug('getTip', { module: 'CardanoNodeClient', value: tip });
            return tip;
        });
    }
    getProtocolParams() {
        return __awaiter(this, void 0, void 0, function* () {
            const protocolParams = yield this.cardanoCli.getProtocolParams();
            this.logger.debug('getProtocolParams', { module: 'CardanoNodeClient', value: protocolParams });
            return protocolParams;
        });
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield fs_extra_1.default.stat(process.env.CARDANO_NODE_SOCKET_PATH);
                if (!(yield this.isInCurrentEra())) {
                    this.logger.warn('cardano-node is still synchronizing', { module: 'CardanoNodeClient' });
                    throw new Error();
                }
            }), {
                factor: 1.5,
                retries: 39,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Establishing connection to cardano-node and ensuring state is in the expected era', this.logger)
            });
        });
    }
    isInCurrentEra() {
        return __awaiter(this, void 0, void 0, function* () {
            const { protocolVersion } = yield this.getProtocolParams();
            this.logger.debug('Comparing current protocol params with last known major version from cardano-node config', {
                module: 'CardanoNodeClient',
                value: {
                    currentProtocolVersion: protocolVersion,
                    lastConfiguredMajorVersion: this.lastConfiguredMajorVersion
                }
            });
            return protocolVersion.major >= this.lastConfiguredMajorVersion;
        });
    }
    submitTransaction(transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const era of util_1.knownEras) {
                const filePath = yield temp_write_1.default(`{
        "type": "${fileTypeFromEra(era)}",
        "description": "",
        "cborHex": "${transaction}"
      }`);
                const hash = util_2.getHashOfSignedTransaction(transaction);
                try {
                    yield this.cardanoCli.submitTransaction(filePath);
                    this.logger.info('submitTransaction', { module: 'CardanoNodeClient', hash: hash });
                    return hash;
                }
                catch (error) {
                    if (!isEraMismatch(error.message)) {
                        throw error;
                    }
                }
                finally {
                    yield fs_extra_1.default.unlink(filePath);
                }
            }
        });
    }
}
exports.CardanoNodeClient = CardanoNodeClient;
//# sourceMappingURL=CardanoNodeClient.js.map