import { Logger } from 'ts-log';
import { Db } from './Db';
import { HasuraClient } from './HasuraClient';
import { AssetWithoutTokens } from './typeAliases';
export interface Signature {
    signature: string;
    publicKey: string;
}
export declare const assetFingerprint: (asset: Pick<AssetWithoutTokens, 'assetName' | 'policyId'>) => string;
export interface AssetMetadata {
    description?: {
        value: string;
        anSignatures: Signature[];
    };
    logo?: {
        value: string;
        anSignatures: Signature[];
    };
    name?: {
        value: string;
        anSignatures: Signature[];
    };
    owner?: Signature;
    preImage?: {
        value: string;
        hashFn: string;
    };
    subject: string;
    ticker?: {
        value: string;
        anSignatures: Signature[];
    };
    url?: {
        value: string;
        anSignatures: Signature[];
    };
}
export declare class DataSyncController {
    readonly hasuraClient: HasuraClient;
    readonly db: Db;
    private logger;
    private metadataServerUri?;
    private axiosClient;
    private assetSynchronizer;
    private metadataSynchronizer;
    constructor(hasuraClient: HasuraClient, db: Db, pollingInterval: {
        initial: number;
        ongoing: number;
    }, logger?: Logger, metadataServerUri?: string);
    private fetchAndApplyMetadata;
    private getAssetMetadata;
    private ensureAssetFingerprints;
    private ensureMetadataServerIsAvailable;
    initialize(): Promise<void>;
    shutdown(): Promise<void>;
}
//# sourceMappingURL=DataSyncController.d.ts.map