import { DataFetcher } from '@cardano-graphql/util';
import { GraphQLSchema } from 'graphql';
import { AdaPots, Asset, AssetSupply, Int_Comparison_Exp as IntComparisonExp, PaymentAddressSummary, ShelleyProtocolParams } from './graphql_types';
import { Logger } from 'ts-log';
import { AssetWithoutTokens } from './typeAliases';
export declare type AdaPotsToCalculateSupply = {
    circulating: AssetSupply['circulating'];
    reserves: AdaPots['reserves'];
};
export declare class HasuraClient {
    readonly hasuraCliPath: string;
    readonly hasuraUri: string;
    readonly lastConfiguredMajorVersion: number;
    private logger;
    private client;
    private applyingSchemaAndMetadata;
    adaPotsToCalculateSupplyFetcher: DataFetcher<AdaPotsToCalculateSupply>;
    currentProtocolVersionFetcher: DataFetcher<ShelleyProtocolParams['protocolVersion']>;
    schema: GraphQLSchema;
    constructor(hasuraCliPath: string, hasuraUri: string, pollingInterval: number, lastConfiguredMajorVersion: number, logger?: Logger);
    private getAdaPotsToCalculateSupply;
    private hasuraCli;
    initialize(): Promise<void>;
    shutdown(): Promise<void>;
    applySchemaAndMetadata(): Promise<void>;
    buildHasuraSchema(): Promise<GraphQLSchema>;
    getCurrentProtocolVersion(): Promise<ShelleyProtocolParams['protocolVersion']>;
    getPaymentAddressSummary(address: string, atBlock?: number): Promise<PaymentAddressSummary>;
    getMeta(nodeTipBlockNumber: number): Promise<{
        initialized: boolean;
        syncPercentage: number;
    }>;
    getDistinctAssetsInTokens(options?: {
        limit: number;
        offset: number;
    }): Promise<AssetWithoutTokens[]>;
    distinctAssetsInTokensCount(): Promise<number>;
    assetsEligibleForMetadataRefreshCount(metadataFetchAttempts: IntComparisonExp): Promise<number>;
    getAssetsIncMetadata(metadataFetchAttempts: IntComparisonExp, options: {
        limit: number;
        offset: number;
    }): Promise<AssetWithoutTokens[]>;
    hasAssetsWithoutFingerprint(): Promise<boolean>;
    getAssetsById(assetIds: Asset['assetId'][]): Promise<AssetWithoutTokens[]>;
    getAssetsWithoutFingerprint(limit?: number): Promise<Pick<AssetWithoutTokens, 'assetId' | 'assetName' | 'policyId'>[]>;
    assetsWithoutMetadataCount(metadataFetchAttempts: IntComparisonExp): Promise<number>;
    getAssetsWithoutMetadata(metadataFetchAttempts: IntComparisonExp, options?: {
        limit: number;
        offset: number;
    }): Promise<AssetWithoutTokens[]>;
    isInCurrentEra(): Promise<boolean>;
    addAssetFingerprints(assets: Pick<AssetWithoutTokens, 'assetId' | 'fingerprint'>[]): Promise<any>;
    addMetadata(assets: (Pick<AssetWithoutTokens, 'assetId' | 'description' | 'logo' | 'name' | 'ticker' | 'url'> & {
        metadataHash: string;
    })[]): Promise<any>;
    incrementMetadataFetchAttempts(assetIds: AssetWithoutTokens['assetId'][]): Promise<any>;
    insertAssets(assets: AssetWithoutTokens[]): Promise<any>;
}
//# sourceMappingURL=HasuraClient.d.ts.map