"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HasuraClient = void 0;
const util_1 = __importStar(require("@cardano-graphql/util"));
const child_process_1 = require("child_process");
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const graphql_1 = require("graphql");
const graphql_request_1 = require("graphql-request");
const wrap_1 = require("@graphql-tools/wrap");
const p_retry_1 = __importDefault(require("p-retry"));
const path_1 = __importDefault(require("path"));
const ts_log_1 = require("ts-log");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
class HasuraClient {
    constructor(hasuraCliPath, hasuraUri, pollingInterval, lastConfiguredMajorVersion, logger = ts_log_1.dummyLogger) {
        this.hasuraCliPath = hasuraCliPath;
        this.hasuraUri = hasuraUri;
        this.lastConfiguredMajorVersion = lastConfiguredMajorVersion;
        this.logger = logger;
        this.applyingSchemaAndMetadata = false;
        this.adaPotsToCalculateSupplyFetcher = new util_1.DataFetcher('AdaPotsToCalculateSupply', () => {
            try {
                return this.getAdaPotsToCalculateSupply();
            }
            catch (error) {
                if (error.message !== 'currentEpoch is only available when close to the chain tip. This is expected during the initial chain-sync.') {
                    throw error;
                }
                this.logger.debug(error.message);
            }
        }, pollingInterval, this.logger);
        this.currentProtocolVersionFetcher = new util_1.DataFetcher('ProtocolParams', () => __awaiter(this, void 0, void 0, function* () {
            const getCurrentProtocolVersion = yield this.getCurrentProtocolVersion();
            this.logger.debug(getCurrentProtocolVersion);
            return getCurrentProtocolVersion;
        }), 1000 * 60, this.logger);
        this.client = new graphql_request_1.GraphQLClient(`${this.hasuraUri}/v1/graphql`, {
            headers: {
                'X-Hasura-Role': 'cardano-graphql'
            }
        });
    }
    getAdaPotsToCalculateSupply() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query {
          cardano { 
              currentEpoch {
                  adaPots {
                      reserves
                  }
              }
          }
          rewards_aggregate {
              aggregate {
                  sum {
                      amount
                  }
              }
          }
          utxos_aggregate {
              aggregate {
                  sum {
                      value
                  }
              }
          }
          withdrawals_aggregate {
              aggregate {
                  sum {
                      amount
                  }
              }
          }
      }`);
            const { cardano, rewards_aggregate: rewardsAggregate, utxos_aggregate: utxosAggregate, withdrawals_aggregate: withdrawalsAggregate } = result;
            const rewards = new bignumber_js_1.default(rewardsAggregate.aggregate.sum.amount);
            const utxos = new bignumber_js_1.default(utxosAggregate.aggregate.sum.value);
            const withdrawals = new bignumber_js_1.default(withdrawalsAggregate.aggregate.sum.amount);
            const withdrawableRewards = rewards.minus(withdrawals);
            return {
                circulating: utxos.plus(withdrawableRewards).toString(),
                reserves: cardano[0].currentEpoch.adaPots.reserves
            };
        });
    }
    hasuraCli(command) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                child_process_1.exec(`${this.hasuraCliPath} --skip-update-check --project ${path_1.default.resolve(__dirname, '..', 'hasura', 'project')} --endpoint ${this.hasuraUri} ${command}`, (error, stdout) => {
                    if (error) {
                        reject(error);
                    }
                    this.logger.debug(stdout);
                    resolve();
                });
            });
        });
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('Initializing', { module: 'HasuraClient' });
            yield this.applySchemaAndMetadata();
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                this.schema = yield this.buildHasuraSchema();
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Fetching Hasura schema via introspection', this.logger)
            });
            this.logger.debug('graphql-engine setup', { module: 'HasuraClient' });
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                const result = yield this.client.request(graphql_request_1.gql `query {
            cardano {
                currentEpoch {
                    number
                }
            }
        }`);
                if (result.cardano.currentEpoch === null) {
                    this.logger.debug('Not in current era');
                    throw new Error('Not in current era');
                }
            }), {
                factor: 1.05,
                retries: 100,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Detecting sync state being in current era', this.logger)
            });
            this.logger.debug('DB is in current era', { module: 'HasuraClient' });
            yield this.currentProtocolVersionFetcher.initialize();
            yield this.adaPotsToCalculateSupplyFetcher.initialize();
            this.logger.info('Initialized', { module: 'HasuraClient' });
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.adaPotsToCalculateSupplyFetcher.shutdown();
            yield this.currentProtocolVersionFetcher.shutdown();
        });
    }
    applySchemaAndMetadata() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.applyingSchemaAndMetadata)
                return;
            this.applyingSchemaAndMetadata = true;
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield this.hasuraCli('migrate apply --down all');
                yield this.hasuraCli('migrate apply --up all');
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Applying PostgreSQL schema migrations', this.logger)
            });
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield this.hasuraCli('metadata clear');
                yield this.hasuraCli('metadata apply');
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Applying Hasura metadata', this.logger)
            });
            this.applyingSchemaAndMetadata = false;
        });
    }
    buildHasuraSchema() {
        return __awaiter(this, void 0, void 0, function* () {
            const executor = ({ document, variables }) => __awaiter(this, void 0, void 0, function* () {
                const query = graphql_1.print(document);
                try {
                    const fetchResult = yield cross_fetch_1.default(`${this.hasuraUri}/v1/graphql`, {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                            'X-Hasura-Role': 'cardano-graphql'
                        },
                        body: JSON.stringify({ query, variables })
                    });
                    return fetchResult.json();
                }
                catch (error) {
                    this.logger.error(error);
                    throw error;
                }
            });
            const coreTypes = [
                'Block',
                'Cardano',
                'Epoch',
                'Block',
                'Transaction'
            ];
            const schema = wrap_1.wrapSchema({
                schema: yield wrap_1.introspectSchema(executor),
                executor
            });
            for (const t of coreTypes) {
                const gqlType = schema.getType(t);
                if (!gqlType) {
                    throw new Error(`Remote schema is missing ${t}`);
                }
            }
            return schema;
        });
    }
    getCurrentProtocolVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query {
          epochs (limit: 1, order_by: { number: desc }) {
              protocolParams {
                  protocolVersion
              }
          }
      }`);
            return result.epochs[0].protocolParams.protocolVersion;
        });
    }
    getPaymentAddressSummary(address, atBlock) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query PaymentAddressSummary (
          $address: String!
          $atBlock: Int
      ){
          utxos (
              where: {
                  _and: {
                      address: { _eq: $address },
                      transaction: { block: { number: { _lte: $atBlock }}}
                  }
              }
          ) {
              value
              tokens {
                  asset {
                      assetId
                      assetName
                      description
                      fingerprint
                      logo
                      metadataHash
                      name
                      ticker
                      tokenMints {
                          quantity
                          transaction {
                              hash
                          }
                      }
                      tokenMints_aggregate {
                          aggregate {
                              avg {
                                  quantity
                              }
                              count
                              max {
                                  quantity
                              }
                              min {
                                  quantity
                              }
                              sum {
                                  quantity
                              }
                          }
                      }
                      url
                      policyId  
                  }
                  quantity
              }
          }
          utxos_aggregate (
              where: {
                  _and: {
                      address: { _eq: $address },
                      transaction: { block: { number: { _lte: $atBlock }}}
                  }
              }
          ) {
              aggregate {
                  count
              }
          }
      }`, {
                address,
                atBlock
            });
            const map = new Map();
            for (const utxo of result.utxos) {
                if (map.has('ada')) {
                    const current = map.get('ada');
                    map.set('ada', Object.assign(Object.assign({}, current), {
                        quantity: new bignumber_js_1.default(current.quantity)
                            .plus(new bignumber_js_1.default(utxo.value))
                            .toString()
                    }));
                }
                else {
                    map.set('ada', {
                        asset: {
                            assetId: 'ada',
                            assetName: 'ada',
                            name: 'ada',
                            policyId: '',
                            tokenMints: [],
                            tokenMints_aggregate: {
                                aggregate: {
                                    avg: {
                                        quantity: 'na'
                                    },
                                    count: 'na',
                                    max: {
                                        quantity: 'na'
                                    },
                                    min: {
                                        quantity: 'na'
                                    },
                                    sum: {
                                        quantity: 'na'
                                    }
                                },
                                nodes: []
                            }
                        },
                        quantity: utxo.value
                    });
                }
                for (const token of utxo.tokens) {
                    if (map.has(token.asset.assetId)) {
                        const current = map.get(token.asset.assetId);
                        map.set(token.asset.assetId, Object.assign(Object.assign({}, current), {
                            quantity: new bignumber_js_1.default(current.quantity)
                                .plus(new bignumber_js_1.default(token.quantity))
                                .toString()
                        }));
                    }
                    else {
                        map.set(token.asset.assetId, token);
                    }
                }
            }
            return {
                assetBalances: [...map.values()],
                utxosCount: result.utxos_aggregate.aggregate.count
            };
        });
    }
    getMeta(nodeTipBlockNumber) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query {
          epochs (limit: 1, order_by: { number: desc }) {
              number
          }
          cardano {
              tip {
                  epoch {
                      number
                  }
                  number
                  forgedAt
              }
          }}`);
            const { tip } = result === null || result === void 0 ? void 0 : result.cardano[0];
            const lastEpoch = result === null || result === void 0 ? void 0 : result.epochs[0];
            return {
                initialized: lastEpoch.number === ((_a = tip.epoch) === null || _a === void 0 ? void 0 : _a.number),
                syncPercentage: (tip.number / nodeTipBlockNumber) * 100
            };
        });
    }
    getDistinctAssetsInTokens(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query DistinctAssetsInTokens (
          $limit: Int
          $offset: Int
      ) {
          tokenMints (
              distinct_on: assetId
              limit: $limit
              order_by: { assetId: asc }
              offset: $offset
          ) {
              assetId
              assetName
              policyId
          }
      }`, {
                limit: options === null || options === void 0 ? void 0 : options.limit,
                offset: options === null || options === void 0 ? void 0 : options.offset
            });
            return result.tokenMints;
        });
    }
    distinctAssetsInTokensCount() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query {
          tokenMints_aggregate (distinct_on: assetId) {
              aggregate {
                  count
              }
          }
      }`);
            return result.tokenMints_aggregate.aggregate.count;
        });
    }
    assetsEligibleForMetadataRefreshCount(metadataFetchAttempts) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield this.client.request(graphql_request_1.gql `query AssetsEligibleForMetadataRefreshCount (
            $metadataFetchAttempts: Int_comparison_exp!
        ) {
            assets_aggregate (
                where: {
                    metadataFetchAttempts: $metadataFetchAttempts
                }) {
                aggregate {
                    count
                }
            }
        }`, {
                    metadataFetchAttempts
                });
                return result.assets_aggregate.aggregate.count;
            }
            catch (error) {
                this.logger.error(error);
                throw error;
            }
        });
    }
    getAssetsIncMetadata(metadataFetchAttempts, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query AssetsIncMetadata (
          $metadataFetchAttempts: Int_comparison_exp
          $limit: Int
          $offset: Int
      ){
          assets (
              limit: $limit
              offset: $offset
              order_by: { assetId: asc }
              where: {
                  metadataFetchAttempts: $metadataFetchAttempts
              }
          ) {
              assetId
              assetName
              description
              name
              policyId
          }
      }`, {
                metadataFetchAttempts,
                limit: options.limit,
                offset: options.offset
            });
            return result.assets;
        });
    }
    hasAssetsWithoutFingerprint() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query {
          assets_aggregate (
              where: { fingerprint: { _is_null: true }}
          ) {
              aggregate {
                  count  
              }
          }
      }`);
            this.logger.debug('Assets without a fingerprint stored', { module: 'HasuraClient', value: result.assets_aggregate.aggregate.count });
            return new bignumber_js_1.default(result.assets_aggregate.aggregate.count).isGreaterThan(0);
        });
    }
    getAssetsById(assetIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query IdsOfAssetsWithoutMetadata (
          $assetIds: [String!]!
      ){
          assets (
              where: {
                  assetId: { _in: $assetIds }
              }) {
              assetId
          }
      }`, {
                assetIds
            });
            return result.assets;
        });
    }
    getAssetsWithoutFingerprint(limit) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query AssetsWithoutFingerprint (
        $limit: Int
      ) {
          assets (
              limit: $limit,
              order_by: { assetId: asc }
              where: { fingerprint: { _is_null: true }}
          ) {
              assetId
              assetName
              policyId
          }
      }`, {
                limit
            });
            return result.assets.map((asset) => (Object.assign(Object.assign({}, asset), { policyId: util_1.default.scalars.Hash28Hex.serialize(asset.policyId) })));
        });
    }
    assetsWithoutMetadataCount(metadataFetchAttempts) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield this.client.request(graphql_request_1.gql `query AssetsWithoutMetadataCount (
            $metadataFetchAttempts: Int_comparison_exp!
        ) {
            assets_aggregate (
                where: {
                    _and: [
                        { metadataFetchAttempts: $metadataFetchAttempts },
                        { metadataHash: { _is_null: true }}
                    ]
                }) {
                aggregate {
                    count
                }
            }
        }`, {
                    metadataFetchAttempts
                });
                return result.assets_aggregate.aggregate.count;
            }
            catch (error) {
                this.logger.error(error);
                throw error;
            }
        });
    }
    getAssetsWithoutMetadata(metadataFetchAttempts, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query IdsOfAssetsWithoutMetadata (
          $limit: Int
          $metadataFetchAttempts: Int_comparison_exp!
          $offset: Int
      ){
          assets (
              limit: $limit
              order_by: { assetId: asc }
              offset: $offset
              where: { 
                  _and: [
                      { metadataFetchAttempts: $metadataFetchAttempts },
                      { metadataHash: { _is_null: true }}
                  ]
              }) {
              assetId
              metadataFetchAttempts
          }
      }`, {
                metadataFetchAttempts,
                limit: options === null || options === void 0 ? void 0 : options.limit,
                offset: options === null || options === void 0 ? void 0 : options.offset
            });
            return result.assets;
        });
    }
    isInCurrentEra() {
        return __awaiter(this, void 0, void 0, function* () {
            const protocolVersion = this.currentProtocolVersionFetcher.value;
            this.logger.debug('Comparing current protocol params with last known major version from cardano-node config', {
                module: 'CardanoNodeClient',
                value: {
                    currentProtocolVersion: protocolVersion,
                    lastConfiguredMajorVersion: protocolVersion.major
                }
            });
            return protocolVersion.major >= this.lastConfiguredMajorVersion;
        });
    }
    addAssetFingerprints(assets) {
        this.logger.debug('Adding fingerprint to assets', { module: 'HasuraClient', value: assets.length });
        return this.client.request(graphql_request_1.gql `mutation AddAssetFingerprint($assets: [Asset_insert_input!]!) {
          insert_assets(
              objects: $assets,
              on_conflict: {
                  constraint: Asset_pkey,
                  update_columns: [fingerprint]
              }
          ) {
              returning {
                  assetId
              }
          }
      }`, {
            assets
        });
    }
    addMetadata(assets) {
        this.logger.info('Adding metadata to assets', { module: 'HasuraClient', value: assets.length });
        return this.client.request(graphql_request_1.gql `mutation AddAssetMetadata($assets: [Asset_insert_input!]!) {
          insert_assets(
              objects: $assets,
              on_conflict: {
                  constraint: Asset_pkey,
                  update_columns: [
                      description,
                      logo,
                      metadataHash,
                      name,
                      ticker,
                      url
                  ]
              }
          ) {
              returning {
                  assetId
              }
          }
      }`, {
            assets
        });
    }
    incrementMetadataFetchAttempts(assetIds) {
        this.logger.debug('Incrementing metadata fetch attempt', { module: 'HasuraClient', value: assetIds.length });
        return this.client.request(graphql_request_1.gql `mutation IncrementAssetMetadataFetchAttempt(
          $assetIds: [String!]!
      ) {
          update_assets(
              where: {
                  assetId: { _in: $assetIds }
              },
              _inc: {
                  metadataFetchAttempts: 1
              }
          ) {
              returning {
                  assetId
                  metadataFetchAttempts
              }
          }
      }`, {
            assetIds
        });
    }
    insertAssets(assets) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug('inserting assets found in tokens', { module: 'HasuraClient', value: assets.length });
            const result = yield this.client.request(graphql_request_1.gql `mutation InsertAssets($assets: [Asset_insert_input!]!) {
        insert_assets(objects: $assets) {
          returning {
            name
            policyId
            description
            assetName
            assetId
          }
          affected_rows
        }
      }`, {
                assets
            });
            return result;
        });
    }
}
exports.HasuraClient = HasuraClient;
//# sourceMappingURL=HasuraClient.js.map