"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSchema = exports.scalarResolvers = void 0;
const fs_1 = __importDefault(require("fs"));
const apollo_server_1 = require("apollo-server");
const schema_1 = require("@graphql-tools/schema");
const delegate_1 = require("@graphql-tools/delegate");
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("@cardano-graphql/util"));
const graphql_scalars_1 = require("graphql-scalars");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const GraphQLBigInt = require('graphql-bigint');
exports.scalarResolvers = {
    AssetFingerprint: util_1.default.scalars.AssetFingerprint,
    BigInt: GraphQLBigInt,
    DateTime: util_1.default.scalars.DateTimeUtcToIso,
    Hash28Hex: util_1.default.scalars.Hash28Hex,
    Hash32Hex: util_1.default.scalars.Hash32Hex,
    IPv4: graphql_scalars_1.IPv4Resolver,
    IPv6: graphql_scalars_1.IPv6Resolver,
    JSON: graphql_scalars_1.JSONResolver,
    JSONObject: graphql_scalars_1.JSONObjectResolver,
    Percentage: util_1.default.scalars.Percentage,
    StakeAddress: util_1.default.scalars.StakeAddress,
    StakePoolID: util_1.default.scalars.StakePoolID,
    Timestamp: graphql_scalars_1.TimestampResolver,
    VRFVerificationKey: util_1.default.scalars.VRFVerificationKey
};
function buildSchema(hasuraClient, genesis, cardanoNodeClient) {
    return __awaiter(this, void 0, void 0, function* () {
        const throwIfNotInCurrentEra = (queryName) => __awaiter(this, void 0, void 0, function* () {
            if (!(yield hasuraClient.isInCurrentEra())) {
                throw new apollo_server_1.ApolloError(`${queryName} results are only available when close to the network tip. This is expected during the initial chain-sync.`);
            }
        });
        return schema_1.makeExecutableSchema({
            resolvers: Object.assign({}, exports.scalarResolvers, {
                Mutation: {
                    submitTransaction: (_root, args) => __awaiter(this, void 0, void 0, function* () {
                        yield throwIfNotInCurrentEra('submitTransaction');
                        const hash = yield cardanoNodeClient.submitTransaction(args.transaction);
                        return { hash };
                    })
                },
                PaymentAddress: {
                    summary: (parent, args) => __awaiter(this, void 0, void 0, function* () {
                        try {
                            return yield hasuraClient.getPaymentAddressSummary(parent.address, args.atBlock);
                        }
                        catch (error) {
                            throw new apollo_server_1.ApolloError(error);
                        }
                    })
                },
                Query: {
                    activeStake: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'activeStake',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    activeStake_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'activeStake_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    ada: () => __awaiter(this, void 0, void 0, function* () {
                        yield throwIfNotInCurrentEra('ada');
                        const adaPots = hasuraClient.adaPotsToCalculateSupplyFetcher.value;
                        if (adaPots === undefined) {
                            return new apollo_server_1.ApolloError('ada query results are not ready yet. This can occur during startup.');
                        }
                        return {
                            supply: {
                                circulating: adaPots.circulating,
                                max: genesis.shelley.maxLovelaceSupply,
                                total: new bignumber_js_1.default(genesis.shelley.maxLovelaceSupply)
                                    .minus(new bignumber_js_1.default(adaPots.reserves))
                                    .toString()
                            }
                        };
                    }),
                    assets: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'assets',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    assets_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'assets_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    blocks: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'blocks',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    blocks_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'blocks_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    cardano: (_root, _args, context, info) => __awaiter(this, void 0, void 0, function* () {
                        try {
                            const result = yield delegate_1.delegateToSchema({
                                context,
                                fieldName: 'cardano',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                            if (result[0].currentEpoch === null) {
                                return new apollo_server_1.ApolloError('currentEpoch is only available when close to the chain tip. This is expected during the initial chain-sync.');
                            }
                            return result[0];
                        }
                        catch (error) {
                            throw new apollo_server_1.ApolloError(error);
                        }
                    }),
                    cardanoDbMeta: () => __awaiter(this, void 0, void 0, function* () {
                        try {
                            const tip = yield cardanoNodeClient.getTip();
                            return hasuraClient.getMeta(tip.blockNo);
                        }
                        catch (error) {
                            throw new apollo_server_1.ApolloError(error);
                        }
                    }),
                    delegations: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'delegations',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    delegations_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'delegations_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    epochs: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'epochs',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    epochs_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'epochs_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    genesis: () => __awaiter(this, void 0, void 0, function* () { return genesis; }),
                    paymentAddresses: (_root, args) => __awaiter(this, void 0, void 0, function* () {
                        yield throwIfNotInCurrentEra('addressSummary');
                        return args.addresses.map((address) => __awaiter(this, void 0, void 0, function* () {
                            return { address };
                        }));
                    }),
                    rewards: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'rewards',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    rewards_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'rewards_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    stakeDeregistrations: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'stakeDeregistrations',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    stakeDeregistrations_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'stakeDeregistrations_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    stakePools: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'stakePools',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    stakePools_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'stakePools_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    stakeRegistrations: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'stakeRegistrations',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    stakeRegistrations_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'stakeRegistrations_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    transactions: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'transactions',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    transactions_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'transactions_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    utxos: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'utxos',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    utxos_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'utxos_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    withdrawals: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'withdrawals',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    },
                    withdrawals_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'withdrawals_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraClient.schema
                        });
                    }
                }
            }),
            typeDefs: fs_1.default.readFileSync(path_1.default.resolve(__dirname, '..', 'schema.graphql'), 'utf-8')
        });
    });
}
exports.buildSchema = buildSchema;
//# sourceMappingURL=executableSchema.js.map